const tableBody = document.getElementById('deviceTableBody');
const progressBar = document.getElementById('scanProgressBar');
const networkrange = document.getElementById('networkrange');
const range1 = document.getElementById('networkrange1');
const range2 = document.getElementById('networkrange2');
var running = false;
var baseIp = "192.168.1"
let knownDevices = [];
const batchSize = 10;

async function fetchWithTimeout(resource, options) {
    const { timeout = 4000 } = options;
    const controller = new AbortController();
    const id = setTimeout(() => controller.abort(), timeout);
    return fetch(resource, {
        ...options,
        signal: controller.signal
    }).then(response => {
        clearTimeout(id);
        return response;
    }).catch(error => {
        clearTimeout(id);
        throw error;
    });
}


function addDeviceToTable(ipAddress, product) {
    const tableBody = document.getElementById('deviceTableBody');
    const row = tableBody.insertRow();

    const cell1 = row.insertCell(0);
    const cell2 = row.insertCell(1);

    // Create a clickable element for the IP address
    const ipElement = document.createElement('a');
    ipElement.href = '#';
    ipElement.textContent = ipAddress;
    ipElement.onclick = function() {
        document.getElementById('inputip').value = ipAddress;
        return false; // Prevent default link behavior
    };

    cell1.appendChild(ipElement);
    cell2.textContent = product;
}

async function checkDevice(ipAddress) {
    try {
        const response = await fetchWithTimeout(`http://${ipAddress}/handshake`, { timeout: 1500 });
        if (!response.ok) throw new Error('Network response was not ok.');

        const data = await response.json();
        if (data && (data.prod == "105-UCON-FREE" || data.prod == "105-UCON-MINI" || data.prod == "105-UCON-HOME" || data.prod == "105-UCON-PRO")) {
            console.log(`unicontrol device found at ${ipAddress}:`, data.prod, data.name);
            knownDevices.push({ ipAddress, product: data.name });
            addDeviceToTable(ipAddress, data.name);
        }
    } catch (error) {}
}

async function net_scan() {
    if(running) return;
    knownDevices = [];
    var fullIp = inputip.value;
    const segments = fullIp.split('.');
    if (segments.length === 4) {
        segments.pop();
        baseIp=segments.join('.');
    } else {
        alert('Invalid IP address format.');
        return;
    }
    running = true;
    progressBar.value = 0;
    range1.innerHTML = baseIp;
    range2.innerHTML = baseIp;
    progressBar.classList.remove("hidden");
    networkrange.classList.remove("hidden");
    tableBody.innerHTML = '';
    for (let i = 0; i <= 255; i += batchSize) {
        const promises = [];
        progressBar.value = i;
        for (let j = i; j < i + batchSize && j <= 255; j++) {
            promises.push(checkDevice(`${baseIp}.${j}`));
        }
        await Promise.allSettled(promises);
    }
    progressBar.classList.add("hidden");
    networkrange.classList.add("hidden");
    running = false;
    savedevices();
}

function savedevices() {
    const expiry = new Date();
    expiry.setTime(expiry.getTime() + (180 * 24 * 60 * 60 * 1000)); // 180 days expiration
    const cookieValue = encodeURIComponent(JSON.stringify(knownDevices));
    document.cookie = `unicontrolscanresults=${cookieValue}; expires=${expiry.toUTCString()}; path=/`;
    localStorage.setItem('unicontrolscanresults', JSON.stringify(knownDevices));
}

function loaddevices() {
    const cookieResult = document.cookie.split('; ').find(row => row.startsWith('unicontrolscanresults='));
    if (cookieResult) {
        const cookieValue = decodeURIComponent(cookieResult.split('=')[1]);
        return JSON.parse(cookieValue);
    } else {
        const savedData = localStorage.getItem('unicontrolscanresults');
        return savedData ? JSON.parse(savedData) : [];
    }
}

function devicesload() {
    const savedDevices = loaddevices();
    savedDevices.forEach(device => addDeviceToTable(device.ipAddress, device.product));
}

function contentfilter(){}
function data_load() {}

document.addEventListener('DOMContentLoaded', devicesload);
